# -*- coding: utf-8 -*-
'''
A module for installing system image base
'''
from __future__ import absolute_import

# Import python libs
import logging
import os
import sys

# Import salt libs
# pylint: disable=import-error,3rd-party-module-not-gated
import salt.utils.files
import salt.utils.platform
import salt.utils.stringutils
import salt.utils.itertools
import salt.modules.cmdmod

from salt.exceptions import CommandExecutionError
# Import local libs
# This file may be loaded out of __pycache__, so the
# directory of its .py may not be in the search path.
IMPORT_PATH = os.path.dirname(__file__)
if IMPORT_PATH.endswith('__pycache__'):
    IMPORT_PATH = os.path.dirname(IMPORT_PATH)
sys.path.append(IMPORT_PATH)

# Define the module's virtual name
__virtualname__ = 'nilrt_system_image'

# Set up logging
log = logging.getLogger(__name__)


def __virtual__():
    if salt.utils.platform.is_windows():
        return (False, 'Module nilrt_system_image: module only works on Linux systems with opkg installed')
    return __virtualname__


def _generate_setimage_blacklists_file(guid):
    content = (
               'etc/natinst/share\n'
               'etc/salt/pki\n'
               'etc/salt/minion_id\n'
               'etc/salt/minion.d/master.conf\n'
               'var/local/natinst/log/niskyline_rt_client_runtime.log\n')
    write_to_file(content, '/.syscfg-action/' + guid + '/SetSystemImageBlacklist')


def _generate_setimage_ini_file(guid):
    content = (
              '[SetImage]\n'
              'Options=4100\n'
              'Timeout=600000\n'
              'LocalPath=/mnt/userfs/.syscfg-action/' + guid + '/systemimage.tar.gz\n'
              'BlackListPath=/mnt/userfs/.syscfg-action/' + guid + '/SetSystemImageBlacklist\n')
    write_to_file(content, '/.syscfg-action/' + guid + '/setimage.ini')


def write_to_file(content, file_path):
    try:
        with salt.utils.files.fopen(file_path, 'w+') as output_file:
            output_file.write(content)
    except IOError:
        return False
    return True


def _call_cmd(cmd):
    '''
    Call cmd.
    '''
    call = __salt__['cmd.run_all'](cmd,
                                   output_loglevel='trace',
                                   ignore_retcode=False,
                                   python_shell=False)

    if call['retcode'] != 0:
        comment = ''
        if call['stderr']:
            comment += call['stderr']
        else:
            comment += call['stdout']

        raise CommandExecutionError(
            '{0}'.format(comment)
        )
    return call

def _remove_file_if_exists(path):
    try:
        if os.path.exists(path):
            os.unlink(path)
    except:
        log.debug('Failed to remove restartcheck leftovers: %s', path)

def _remove_restartcheck_leftover_files_hack():
    _remove_file_if_exists('/var/volatile/tmp/uImage')
    _remove_file_if_exists('/var/volatile/tmp/uImage.gz')

def install(system_image_pkg_name, system_image_pkg_version, **kwargs):
    '''
    Installs the system base image.

    :param system_image_pkg_name: The system base image package name
    :type system_image_pkg_name: str
    :param system_image_pkg_version: The system base image package version
    :type system_image_pkg_version: str
    '''
    if ('nisystemlink_version' not in __grains__) or (__grains__['nisystemlink_version'].startswith('18')):
        raise Exception('The system image is supported only on NI Linux RT targets with SystemLink Client 19.0 or higher.')

    # These leftovers are 10MB and since on ARM targets the space is
    # already tight those leftovers can make the installation fail
    _remove_restartcheck_leftover_files_hack()

    refreshdb = kwargs.get('refresh', False)
    __salt__['pkg.install'](system_image_pkg_name, refresh=refreshdb, version=system_image_pkg_version, reinstall=True)
    provides_result = __salt__['pkg.info_installed'](system_image_pkg_name, attr='provides')[system_image_pkg_name]['provides']
    guid = provides_result.split(' ')[0]
    _generate_setimage_blacklists_file(guid)
    _generate_setimage_ini_file(guid)
    _call_cmd(['/etc/init.d/nisetbootmode', 'force-install'])
    __salt__['nisysmgmt.restart']()
