# -*- coding: utf-8 -*-
'''
A state module for installing system image base when applying a state
'''
from __future__ import absolute_import

# Import python libs
import os

# Import salt libs
import salt.utils.files
import salt.utils.itertools


def installed(name, version, reinstall=False, **kwargs):
    '''
    Installs the system base image.

    :param name: The system base image package name
    :type name: str
    :param version: The system base image package major version
    :type version: str
    :param reinstall: An optional flag that indicates if the image is already installed it should be reinstalled or not
    :type reinstall: bool
    '''
    ret = {"name": name, "changes": {}, "result": None, "comment": ""}

    if _is_testmode(**kwargs):
        ret["comment"] = {
            name: "System image will be installed."
        }
        return ret

    path_to_installing_flag = "/etc/natinst/share/imageInstalled"
    if not os.path.exists(path_to_installing_flag):
        installed_version = __grains__.get("nilrt_system_image_version")
        if installed_version and installed_version in version and not reinstall:
            ret["comment"] = "System image already installed."
            remove_upgrade_tools(**kwargs)
            return ret

        full_version_to_install = get_full_version_to_install(name, version)
        mark_installation_in_progress(path_to_installing_flag)
        try:
            __salt__["nilrt_system_image.install"](name, full_version_to_install, **kwargs)
        except:
            mark_installation_finished(path_to_installing_flag)
            remove_upgrade_tools(**kwargs)
            raise
    else:
        mark_installation_finished(path_to_installing_flag)
        remove_upgrade_tools(**kwargs)
        ret["changes"] = {
            name: "Successfully installed."
        }
    return ret


def _is_testmode(**kwargs):
    '''
    Returns whether a test mode (noaction) operation was requested.
    '''
    return bool(kwargs.get('test') or __opts__.get('test'))


def get_full_version_to_install(name, version):
    '''
    Returns the full version of the image that is going to be installed if it is available on the configured feeds.

    :param name: The system base image name
    :type name: str
    :param version: The system base image package major version
    :type version: str
    '''
    cmd = ["opkg", "list", name]
    available_versions_info = __salt__["cmd.run_stdout"](cmd, output_loglevel="trace", python_shell=False)
    for line in salt.utils.itertools.split(available_versions_info, "\n"):
        full_version = line.split(" - ")[1]
        if version in full_version:
            return full_version
    return version


def mark_installation_in_progress(flag_path):
    '''
    Creates a file that indicates that the image is being installed

    :param flag_path: The path to the flag file
    :type flag_path: str
    '''
    with salt.utils.files.fopen(flag_path, "x"):
        return

def remove_upgrade_tools(**kwargs):
    '''
    Removes the upgrade tools

    '''
    __salt__["pkg.remove"](pkgs=['dist-nilrt-upgrade-tools'], **kwargs)

def mark_installation_finished(flag_path):
    '''
    Deletes the file that indicates that the image is being installed

    :param flag_path: The path to the flag file
    :type flag_path: str
    '''
    salt.utils.files.safe_rm(flag_path)
